<?php namespace Pastee\Model;

use Blocktrail\CryptoJSAES\CryptoJSAES;
use Carbon\Carbon;

class Paste {
	public $id;

	public $description;

	public $encrypted;

	public $views;

	public $sections;

	public $created_at;

	public $expires_at = null;

	public function __construct($description = '', $sections = [], $options = []) {
		$this->description = $description;
		$this->sections = $sections;
	}

	public function withDescription($description) {
		$this->description = $description;
		return $this;
	}

	public function withSection($section, $contents = '', $syntax = 'autodetect') {
		if (!($section instanceof PasteSection)) {
			$section = new PasteSection($section, $syntax, $contents);
		}
		$this->sections[] = $section;
		return $this;
	}

	public function withSections($sections) {
		$this->sections = $sections;
		return $this;
	}

    /**
     * Decrypt the paste using CryptoJS AES compatible code.
     *
     * @param $key
     * @return $this
     * @throws \Exception
     */
    public function decrypt($key) {
		if (strlen($key) < 32) {
			throw new \Exception('Invalid key length!');
		}

		foreach ($this->sections as $section) {
			$section->contents = CryptoJSAES::decrypt($section->contents, $key);
		}

		return $this;
	}

	public function encrypt($key = null) {
		if (is_null($key)) {
			// TODO generate key
		}

		if (strlen($key) < 32) {
			throw new \Exception('Invalid key length!');
		}

		foreach ($this->sections as $section) {
			$section->contents = CryptoJSAES::encrypt($section->contents, $key);
		}

		return $this;
	}

	public static function fromObject($json) {
		$paste = new Paste();

		$paste->id = $json->paste->id;
		$paste->description = $json->paste->description;
		$paste->encrypted = $json->paste->encrypted;
		$paste->views = $json->paste->views;
		$paste->created_at = Carbon::parse($json->paste->created_at);

		if ($json->paste->expires_at != null) {
			$paste->expires_at = Carbon::parse($json->paste->expires_at);
		}

		foreach ($json->paste->sections as $section) {
			$paste->sections[$section->id] = new PasteSection($section->name, $section->syntax, $section->contents);
		}

		return $paste;
	}
}